@@ -38,6 +38,7 @@
 #include "yelp-reader.h"
 #include "yelp-scrollkeeper.h"
 #include "yelp-util.h"
+#include "yelp-uri.h"
 #include "yelp-view-content.h"
 
 #define d(x)
@@ -50,24 +51,26 @@ static void content_html_uri_selected_cb      (YelpHtml             *html,
 					       YelpURI              *uri,
 					       gboolean              handled,
 					       YelpViewContent      *view);
-#if 0
-static void content_reader_start_cb           (YelpReader           *reader,
+static void content_html_title_changed_cb     (YelpHtml             *html,
+					       const gchar          *title,
 					       YelpViewContent      *view);
-#endif
 static void content_reader_data_cb            (YelpReader           *reader,
 					       const gchar          *data,
 					       gint                  len,
 					       YelpViewContent      *view);
 static void content_reader_finished_cb        (YelpReader           *reader,
+					       YelpURI              *uri,
 					       YelpViewContent      *view);
 static void content_reader_error_cb           (YelpReader           *reader,
 					       GError               *error,
 					       YelpViewContent      *view);
-
-
+static GtkTreePath *
+content_find_path_from_uri                    (GtkTreeModel         *model,
+					       YelpURI              *uri);
 
 enum {
 	URI_SELECTED,
+	TITLE_CHANGED,
 	LAST_SIGNAL
 };
 
@@ -88,7 +91,7 @@ struct _YelpViewContentPriv {
 
 	YelpURI      *current_uri;
 	
-	gboolean      loading;
+	gboolean      first;
 };
 
 GType
@@ -140,19 +143,17 @@ content_init (YelpViewContent *view)
 	priv->html_widget     = yelp_html_get_widget (priv->html_view);
 /* 	priv->current_docpath = g_strdup (""); */
 	priv->current_uri     = NULL;
-	priv->loading         = FALSE;
+	priv->first         = FALSE;
 	
 	g_signal_connect (priv->html_view, "uri_selected",
 			  G_CALLBACK (content_html_uri_selected_cb), 
 			  view);
+	g_signal_connect (priv->html_view, "title_changed",
+			  G_CALLBACK (content_html_title_changed_cb),
+			  view);
 
 	priv->reader      = yelp_reader_new ();
 	
-#if 0
-	g_signal_connect (G_OBJECT (priv->reader), "start",
-			  G_CALLBACK (content_reader_start_cb),
-			  view);
-#endif
 	g_signal_connect (G_OBJECT (priv->reader), "data",
 			  G_CALLBACK (content_reader_data_cb),
 			  view);
@@ -177,6 +178,17 @@ content_class_init (YelpViewContentClass *klass)
 			      yelp_marshal_VOID__POINTER_BOOLEAN,
 			      G_TYPE_NONE,
 			      2, G_TYPE_POINTER, G_TYPE_BOOLEAN);
+
+	signals[TITLE_CHANGED] = 
+		g_signal_new ("title_changed",
+			      G_TYPE_FROM_CLASS (klass),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (YelpViewContentClass,
+					       title_changed),
+			      NULL, NULL,
+			      yelp_marshal_VOID__STRING,
+			      G_TYPE_NONE,
+			      1, G_TYPE_STRING);
 }
 
 static void
@@ -193,9 +205,17 @@ content_html_uri_selected_cb (YelpHtml        *html,
 	g_signal_emit (view, signals[URI_SELECTED], 0, uri, handled);
 }
 
+static void
+content_html_title_changed_cb (YelpHtml        *html,
+			       const gchar     *title,
+			       YelpViewContent *view)
+{
+	g_signal_emit (view, signals[TITLE_CHANGED], 0, title);
+}
+
 static void
 content_tree_selection_changed_cb (GtkTreeSelection *selection, 
-			       YelpViewContent  *content)
+				   YelpViewContent  *content)
 {
 	YelpViewContentPriv *priv;
  	GtkTreeIter          iter;
@@ -219,38 +239,6 @@ content_tree_selection_changed_cb (GtkTreeSelection *selection,
 	}
 }
 
-
-#if 0
-static void
-content_reader_start_cb (YelpReader *reader, YelpViewContent *view)
-{
-	GdkCursor           *cursor;
-	YelpViewContentPriv *priv;
-	gchar               *loading = _("Loading...");
-
-	g_return_if_fail (YELP_IS_READER (reader));
-	g_return_if_fail (YELP_IS_VIEW_CONTENT (view));
-
-	priv = view->priv;
-
-	d(g_print ("Start_cb\n"));
-	
-	cursor = gdk_cursor_new (GDK_WATCH);
-	
-	gdk_window_set_cursor (priv->html_widget->window, cursor);
-	gdk_cursor_unref (cursor);
-	
-	yelp_html_clear (priv->html_view);
-	
-	priv->loading = TRUE;
-	
-	yelp_html_printf (priv->html_view, 
-			  "<html><meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\"><body bgcolor=\"white\"><center>%s</center></body></html>", 
-			  loading);
- 	yelp_html_close (priv->html_view);
-}
-#endif
-
 static void
 content_reader_data_cb (YelpReader      *reader,
 			const gchar     *data,
@@ -264,9 +252,9 @@ content_reader_data_cb (YelpReader      *reader,
 	
 	priv = view->priv;
 
-	if (priv->loading) {
+	if (priv->first) {
 		yelp_html_clear (priv->html_view);
-		priv->loading = FALSE;
+		priv->first = FALSE;
 	}
 
 	if (len == -1) {
@@ -281,16 +269,69 @@ content_reader_data_cb (YelpReader      *reader,
 }
 
 static void
-content_reader_finished_cb (YelpReader *reader, YelpViewContent *view)
+content_reader_finished_cb (YelpReader      *reader, 
+			    YelpURI         *uri,
+			    YelpViewContent *view)
 {
 	YelpViewContentPriv *priv;
+	GtkTreePath         *path = NULL;
 	
 	g_return_if_fail (YELP_IS_READER (reader));
 	g_return_if_fail (YELP_IS_VIEW_CONTENT (view));
 
 	priv = view->priv;
 
-	yelp_html_close (priv->html_view);
+	if (!priv->first) {
+		yelp_html_close (priv->html_view);
+	}
+
+	path = content_find_path_from_uri (GTK_TREE_MODEL (priv->tree_store),
+					   uri);
+	
+	if (path) {
+		GtkTreeSelection *selection;
+		GtkTreePath      *parent;
+
+		/* Open the correct node in the tree */
+
+		d(g_print ("Found path\n"));
+		
+		selection = gtk_tree_view_get_selection (
+			GTK_TREE_VIEW (priv->content_tree));
+			
+ 		parent = gtk_tree_path_copy (path);
+		
+		gtk_tree_path_up (parent);
+
+		gtk_tree_view_expand_row (GTK_TREE_VIEW (priv->content_tree),
+ 					  parent, FALSE);
+
+		gtk_tree_view_expand_row (GTK_TREE_VIEW (priv->content_tree),
+ 					  path, TRUE);
+
+		parent = gtk_tree_path_copy (path);
+		
+		g_signal_handlers_block_by_func (
+			selection,
+			content_tree_selection_changed_cb,
+			view);
+ 
+		gtk_tree_selection_select_path (selection, path);
+		
+		g_signal_handlers_unblock_by_func (
+			selection,
+			content_tree_selection_changed_cb,
+			view);
+
+		while (gtk_tree_path_up (path)) {
+			gtk_tree_view_expand_row (
+				GTK_TREE_VIEW (priv->content_tree),
+				path, FALSE);
+		}
+
+		gtk_tree_path_free (path);
+		gtk_tree_path_free (parent);
+	}
 
 	gdk_window_set_cursor (priv->html_widget->window, NULL);
 	gtk_widget_grab_focus (priv->html_widget);
@@ -313,6 +354,47 @@ content_reader_error_cb (YelpReader      *reader,
 	g_warning ("%s\n", error->message);
 }
 
+static GtkTreePath *found_path;
+
+static gboolean
+content_tree_model_foreach (GtkTreeModel *model,
+			    GtkTreePath  *path,
+			    GtkTreeIter  *iter,
+			    YelpURI      *uri)
+{
+	YelpSection *section = NULL;
+	
+	g_return_val_if_fail (GTK_IS_TREE_MODEL (model), TRUE);
+	g_return_val_if_fail (uri != NULL, TRUE);
+	
+	gtk_tree_model_get (model, iter, 
+			    1, &section, 
+			    -1);
+
+	if (!section) {
+		return FALSE;
+	}
+	
+	if (yelp_uri_equal (uri, section->uri)) {
+		found_path = gtk_tree_path_copy (path);
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
+static GtkTreePath *
+content_find_path_from_uri (GtkTreeModel *model, YelpURI *uri)
+{
+	found_path = NULL;
+	
+	gtk_tree_model_foreach (model, 
+				(GtkTreeModelForeachFunc) content_tree_model_foreach,
+				uri);
+
+	return found_path;
+}
+
 GtkWidget *
 yelp_view_content_new (GNode *doc_tree)
 {
@@ -454,7 +536,7 @@ yelp_view_content_show_uri (YelpViewContent  *content,
 		
 	}
 	
-	priv->loading = TRUE;
+	priv->first = TRUE;
 	
 	cursor = gdk_cursor_new (GDK_WATCH);
 		
